#include <AFMotor.h>
#include <NewPing.h>
#include <Servo.h>

// Pins and configuration
#define TRIG_PIN A0
#define ECHO_PIN A1
#define MAX_DISTANCE 200
#define MAX_SPEED 150

#define HEAD_CENTER 100
#define HEAD_LEFT 170
#define HEAD_RIGHT 50

int motor1Speed = 155;
int motor2Speed = 125;

NewPing sonar(TRIG_PIN, ECHO_PIN, MAX_DISTANCE);
AF_DCMotor motor1(1, MOTOR12_1KHZ);
AF_DCMotor motor2(2, MOTOR12_1KHZ);

Servo headServo;
Servo leftArm;
Servo rightArm;

boolean goesForward = false;
int distance = 100;
bool armState = false;
unsigned long lastArmMoveTime = 0;
const int armMoveInterval = 400;

// Corner escape logic
int cornerEscapeStage = 0;
bool hasWaved = false;

void setup() {
  headServo.attach(10);
  leftArm.attach(9);
  rightArm.attach(A2);

  headServo.write(HEAD_CENTER);
  leftArm.write(90);
  rightArm.write(90);
  delay(1000);  // Let servos settle
}

void loop() {
  if (!hasWaved) {
    waveRightArm();       // 👋 One-time wave
    rightArm.write(90);   // Return to rest
    hasWaved = true;
    delay(1000);
    return;               // Skip rest of loop once
  }

  delay(40);
  distance = readPing();

  if (distance <= 15) {
    moveStop();
    stopArms();
    delay(100);
    moveBackward();
    delay(300);
    moveStop();
    delay(200);

    int distanceR = lookRight();
    delay(200);
    int distanceL = lookLeft();
    delay(200);

    cornerEscapeStage++;

    if (cornerEscapeStage == 1) {
      if (distanceR >= distanceL) {
        turnRight();
      } else {
        turnLeft();
      }
    } else if (cornerEscapeStage == 2) {
      if (distanceR < distanceL) {
        turnRight();
      } else {
        turnLeft();
      }
    } else {
      performEscapeTurn();  // 180° turn
      cornerEscapeStage = 0;
    }

    moveStop();
  } else {
    moveForward();
    marchArms();
    cornerEscapeStage = 0;
  }
}

// 👋 Wave with right arm
void waveRightArm() {
  for (int i = 0; i < 2; i++) {
    rightArm.write(120); delay(500);
    rightArm.write(60);  delay(500);
  }
}

// 🚶‍♂️ Arm marching
void marchArms() {
  unsigned long currentMillis = millis();
  if (currentMillis - lastArmMoveTime >= armMoveInterval) {
    armState = !armState;
    if (armState) {
      leftArm.write(120);
      rightArm.write(120);
    } else {
      leftArm.write(60);
      rightArm.write(60);
    }
    lastArmMoveTime = currentMillis;
  }
}

// 🛑 Stop arms
void stopArms() {
  leftArm.write(90);
  rightArm.write(90);
}

// 👁️ Look right
int lookRight() {
  headServo.write(HEAD_RIGHT);
  delay(500);
  int d = readPing();
  headServo.write(HEAD_CENTER);
  return d;
}

// 👁️ Look left
int lookLeft() {
  headServo.write(HEAD_LEFT);
  delay(500);
  int d = readPing();
  headServo.write(HEAD_CENTER);
  return d;
}

// 📏 Read ping
int readPing() {
  delay(70);
  int cm = sonar.ping_cm();
  return cm == 0 ? 250 : cm;
}

// 🛑 Stop motors
void moveStop() {
  motor1.run(RELEASE);
  motor2.run(RELEASE);
  goesForward = false;
}

// ▶️ Move forward
void moveForward() {
  if (!goesForward) {
    goesForward = true;
    motor1.run(FORWARD);
    motor2.run(FORWARD);
  }
  motor1.setSpeed(motor1Speed);
  motor2.setSpeed(motor2Speed);
}

// ◀️ Move backward
void moveBackward() {
  goesForward = false;
  motor1.run(BACKWARD);
  motor2.run(BACKWARD);
  motor1.setSpeed(motor1Speed);
  motor2.setSpeed(motor2Speed);
}

// ↪️ Turn right
void turnRight() {
  motor1.run(FORWARD);
  motor2.run(BACKWARD);
  motor1.setSpeed(motor1Speed);
  motor2.setSpeed(motor2Speed);
  delay(400);
  moveStop();
}

// ↩️ Turn left
void turnLeft() {
  motor1.run(BACKWARD);
  motor2.run(FORWARD);
  motor1.setSpeed(motor1Speed);
  motor2.setSpeed(motor2Speed);
  delay(400);
  moveStop();
}

// 🔁 180° Turn
void performEscapeTurn() {
  motor1.run(FORWARD);
  motor2.run(BACKWARD);
  motor1.setSpeed(motor1Speed);
  motor2.setSpeed(motor2Speed);
  delay(800);
  moveStop();
}
